/*
 * Copyright (c) 2003 Linuxant inc.
 * Copyright (c) 2001-2003 Conexant Systems, Inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

/****************************************************************************************
*		                     Version Control Information
*
*	$Header:   P:/d942/octopus/archives/linuxos/dist/modules/osspec/osnvm.c_v   1.4   03 Jan 2003 10:57:26   beliloi  $
*
*****************************************************************************************/

/****************************************************************************************

  File Name:			OsNVM.c

  File Description:		This file implements the Os dependent part of the NVM Manager.

*****************************************************************************************/
#define LINUX_NVM

#include "oscompat.h"
#include "ostypedefs.h"
#include "comtypes.h"
#include "configtypes.h"
#include "osservices.h"
#include "osstdio.h"
#include "osmemory.h"
#include "osnvm.h"

//#include "testdebug.h"


#define __KERNEL_SYSCALLS__
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/unistd.h>
#include <asm/fcntl.h>
#include <asm/uaccess.h>

static int errno;
static inline _syscall2 (int, rename, const char *, old, const char *, new);

static int
OsRenameFile (const char *old, const char *new)
{
	mm_segment_t fs = get_fs ();
	int fh;

	set_fs (get_ds ());
	fh = rename (old, new);
	set_fs (fs);
	if(fh)
		return errno;
	else
		return fh;
}

#define CFILE_BUFFER_SIZE	4096
typedef struct tagCharFile {
	FILE *file;
	int size;
	int len;
	UINT32 position;
	char *pcur;
	char buf[CFILE_BUFFER_SIZE];
} CHAR_FILE;

typedef struct tagSection {
	UINT32 dwPosition;
	char name[20];
} NVM_SECTION;

typedef enum {
	Modem_Section = 0,
	OEM_Section,
	Country_Section,
	Cty_Section,
	Num_Section
} INF_SECTION;

typedef struct tagNVM {
	CHAR_FILE inf_file;
	CHAR_FILE cty_file;
	CHAR_FILE new_file;
	NVM_SECTION sectionList[Num_Section];
} NVM_HANDLE;

/*
NVM_SECTION Modem_Section =
{
	0x0,
	"[Modem_HW]"
};

NVM_SECTION OEM_Section =
{
	0x0,
	"[OEM]"
};

NVM_SECTION Country_Section =
{
	0x0,
	"[Country]"
};

NVM_SECTION Cty_Section =
{
	0x0,
	"[Country]"
};
*/

static const PROFILE_DATA g_FactoryProfile = {
	1,							// Echo
	1,							// Volume
	1,							// Speaker control
	0,							// Pulse
	0,							// Quiet
	1,							// Verbose
	4,							// report Level
	0,							// Connect message
	1,							// AmperC
	2,							// AmperD
	0,							// S0
	0,							// S1;
	43,							// S2;
	13,							// S3;
	10,							// S4;
	8,							// S5;
	2,							// S6;
	50,							// S7;
	2,							// S8;
	14,							// S10;
	50,							// S12;
	0,							// S16;
	0,							// S18;
	70							// S29;
};

static const char inf_name[] = HCFINF_FILE;
static const char new_name[] = HCFINF_FILE ".new";
static const char old_name[] = HCFINF_FILE ".old";
static const char cty_name[] = HCFCTY_FILE;

/****************************************************************************************

Function Name:			NVM_Open

Function Description:	Get device Node key from Registry.

Arguments:				dwDevNode

Return Value:			Handle to the device node.

Notes:					

*****************************************************************************************/
HANDLE
NVM_Open (UINT32 dwDevNode)
{
#ifdef LINUX_NVM
	NVM_HANDLE *h;

//  printk("NVM_Open\n");
	if ((h = OsAllocate (sizeof (NVM_HANDLE))) == NULL) {
		return NULL;
	}

	h->inf_file.file = h->cty_file.file = h->new_file.file = (FILE *) NULL;
/*  if((h->inf_file.file == NULL) || (h->cty_file.file == NULL))
  {
  	NVM_Close((HANDLE) h);
		printk("NVM: Can't open files\n");
		return NULL;
  }
*/
	h->sectionList[Modem_Section].dwPosition = 0;
	OsStrCpy (h->sectionList[Modem_Section].name, "[Modem_HW]");
	h->sectionList[OEM_Section].dwPosition = 0;
	OsStrCpy (h->sectionList[OEM_Section].name, "[OEM]");
	h->sectionList[Country_Section].dwPosition = 0;
	OsStrCpy (h->sectionList[Country_Section].name, "[Country]");
	h->sectionList[Cty_Section].dwPosition = 0;
	OsStrCpy (h->sectionList[Cty_Section].name, "[Country]");
	return (HANDLE) h;
#else
	return (HANDLE) 0xFFFF;
#endif
}


/****************************************************************************************

Function Name:			NVM_Close

Function Description:	

Arguments:				hNVM - NVM handle

Return Value:			COM_STATUS_SUCCESS

Notes:					

*****************************************************************************************/
COM_STATUS
NVM_Close (HANDLE hNVM)
{
#ifdef LINUX_NVM
//  printk("NVM_Close\n");
	if (hNVM != NULL) {
		if (((NVM_HANDLE *) hNVM)->inf_file.file != NULL) {
			OsFClose (((NVM_HANDLE *) hNVM)->inf_file.file);
			((NVM_HANDLE *) hNVM)->inf_file.file = NULL;
		}
		if (((NVM_HANDLE *) hNVM)->cty_file.file != NULL) {
			OsFClose (((NVM_HANDLE *) hNVM)->cty_file.file);
			((NVM_HANDLE *) hNVM)->cty_file.file = NULL;
		}
		OsFree ((PVOID) hNVM);
	}
#endif
	return COM_STATUS_SUCCESS;
}

void
NVM_rewind (CHAR_FILE * h, UINT32 position)
{
//  printk("NVM_rewind start\n");
	h->size = h->len = 0;
	h->pcur = h->buf;
	h->buf[0] = '\0';
	h->position = position;
	if (h->file)
		OsFSeek (h->file, position, SEEK_SET);
//  printk("NVM_rewind end\n");
}

void
NVM_copy (CHAR_FILE * fdst, CHAR_FILE * fsrc, UINT32 startPos, UINT32 endPos)
{
	UINT32 size;
	int len;
//  printk("NVM_copy: %d %d\n", startPos, endPos);
	NVM_rewind (fsrc, startPos);
	while (startPos < endPos) {
		size = endPos - startPos;
		len = (size > CFILE_BUFFER_SIZE - 1) ? CFILE_BUFFER_SIZE - 1 : size;
//      printk("NVM_copy before: %d\n", len);
		len = OsFRead (fsrc->buf, 1, len, fsrc->file);
		if(len == 0) {
//			printk(__FUNCTION__": premature eof\n");
			break;
		}
/*		
		if(len >= 0)
		{
				fsrc->buf[len] = '\0';
				printk("%s", fsrc->buf);
		}
*/
		if (OsFWrite (fsrc->buf, 1, len, fdst->file) != len) {
//          printk("NVM_copy ERROR: %d\n", len);
			break;
		}
		startPos += (UINT32) len;
	}
}

void
NVM_SubstFile (CHAR_FILE * dest, CHAR_FILE * src)
{
	int ret;
	if (dest->file)
		OsFClose (dest->file);
	if (src->file)
		OsFClose (src->file);
	dest->file = src->file = NULL;
	OsRenameFile (inf_name, old_name);
	if ((ret = OsRenameFile (new_name, inf_name))) {
//          printk("NVM_SubstFile: Can't rename: %d\n", ret);
	}
//  dest->file = OsFOpen(inf_name);
//  printk("NVM_SubstFile: NEW FILE: %d\n", dest->file);    
}

/****************************************************************************************

Function Name:			NMV_gets

Function Description:	

Arguments:				h - handle of char file

Return Value:			pointer to the string

Notes:					  output points to zero-terminated string located in buffer insite insite CHAR_HANDLE structure

*****************************************************************************************/
char *
NVM_gets (CHAR_FILE * h)
{
	char *pch, termchar, *ret = NULL;
	int len;

//  printk("NVM_gets\n");
	while (1) {
		if ((pch = strchr (h->pcur, '\n')) != NULL) {
			h->position += h->len;
			*pch = '\0';
			ret = h->pcur;
			h->len = OsStrLen (ret) + 1;
			if ((h->size -= h->len) < 0)
				h->size = 0;
			h->pcur = pch + 1;

			if ((pch = strchr (ret, ';')) != NULL) {
				*pch = '\0';
			}
//          printk("NVM_gets(%x : %x): %s\n", h->position, h->size, (ret)?ret:"");
//          printk("NVM_gets: %ld\n", h->position);
			break;
		} else {
			int readsize;
//          printk("READ: %d %d\n", h->size, OsStrLen(h->pcur));
			if (h->size > 0) {
/*				if(h->size >= CFILE_BUFFER_SIZE-1)
				{
					printk("\n\nERROR!!!!! %d\n\n\n", OsStrLen(h->buf));
					h->buf[OsStrLen(h->buf)] = ' ';
					continue;
				}*/
//              printk("COPY: %d %d\n", h->size, OsStrLen(h->pcur));
				OsMemCpy (h->buf, h->pcur, h->size);
			} else
				h->size = 0;
			readsize = CFILE_BUFFER_SIZE - 1 - h->size;
			len = OsFRead (h->buf + h->size, 1, readsize, h->file);
			if (len <= 0) {
// EOF or error
//              printk("NVM: EOF or error is occured on %d: %d\n", h->file, len);
				break;
			}
			termchar = (len < readsize) ? '\n' : '\0';
			h->size += len;
			h->buf[h->size] = termchar;
			h->pcur = h->buf;
//          printk("OsFRead: %p(%p) %d : %d (%d)\n", h->buf+h->size, h->buf, readsize, len, h->size);
//          printk("NVM_gets: %s\n", h->buf);
		}
	}
	return ret;
}


/****************************************************************************************

Function Name:		NVM_GetField

Function Description:	

Arguments:				f   	- handle of char file
									str 	- pointer to current position in input string
									field	- pointer to buffer the result should be stored

Return Value:			pointer to current position in input string - if there are more fields in the string
									NULL - it there are no more parameter for current subkey

Notes:					  

*****************************************************************************************/
char *
NVM_GetField (CHAR_FILE * f, char *str, char *field)
{
	int len = 0;
	int inquote;
	char *ret = NULL;

//  printk("NVM_GetField\n");
	while (1) {
		if (str == NULL) {
			if ((str = NVM_gets (f)) == NULL)
				break;
		}
		for (; (*str <= ' ') && (*str != '\0'); str++);
//  printk("%s\n", str);

		for (len = 0, inquote = FALSE; ; str++) {
			if(inquote) {
				if(*str == '"')
					inquote = FALSE;

				if(*str < ' ')
					break;
			} else {
				if(*str == '"')
					inquote = TRUE;
				if ((*str <= ' ') || (*str == ','))
					break;
			}

			field[len++] = *str;
		}
		if (field[0] != '\\')
			break;
		str = NULL;
	}
	field[len] = '\0';
//  printk("F: %s\n", field);
	if (str != NULL)
		ret = strchr (str, ',');
	if (ret != NULL)
		ret++;
	return ret;
}

int
NVM_FindSection (CHAR_FILE * f, NVM_SECTION * section)
{
	char *p, str[80];
	int ret = 0;
//	printk("NVM_FindSection: %s (%ld)- \n", section->name, section->dwPosition);
//  if(section->dwPosition != 0x0L)
	NVM_rewind (f, section->dwPosition);
	while ((p = NVM_gets (f))) {
// Find the section
		if ((p = NVM_GetField (f, p, str)) == NULL) {
//              printk("s:'%s' s:'%s'\n", str, section->name);
			if (!OsStrCmp (str, section->name)) {
				section->dwPosition = f->position;
//                  printk("NVM_FindSection: '%s' found at %ld\n", section->name, section->dwPosition);
				ret = 1;
				break;
			}
		}
		continue;
	}
//	printk("NVM_FindSection: %s\n",(ret)? "found":"not exist");
	return ret;
}

/****************************************************************************************

Function Name:		NVM_FindSubKey

Function Description:	

Arguments:				f   	- handle of char file
									subkey- pointer to subkey that sould be found

Return Value:			pointer to the next field in input string - if there are more fields in the string
									NULL - no more subkeys (strings) in the file

Notes:					  

*****************************************************************************************/

char *
NVM_FindSubKey (CHAR_FILE * f, char *subkey)
{
	char *p, *ret = NULL, str[80];

//	printk("NVM_FindSubKey: %s - \n", subkey);
	while (1) {
		p = NVM_gets (f);
		if (p != NULL) {
// Get first field in string        
			if ((p = NVM_GetField (f, p, str)) == NULL) {
				if (*str != '[')
					continue;
//              printk("NVM_FindSubKey: No such key in the section\n");
				break;
			}
// Is the field "HKR"   
			if (OsStrCmp (str, "HKR"))
				continue;
// Get next field   
			if ((p = NVM_GetField (f, p, str)) == NULL)
				continue;
// Is the field subkey
			if (OsStrCmp (str, subkey))
				continue;
//          printk("%s\n", p);
			ret = p;
/*			
			if((ret = strstr(p, subkey)) == NULL)
				continue;
*/
		}
		break;
/*		if((ret = strchr(ret, ',')) != NULL)
			break;
*/
	}
//	printk("NVM_FindSubKey: '%s' '%s' %s\n",str, subkey, (ret)? "found":"not exist");
	return ret;
}

/****************************************************************************************

Function Name:		NVM_ReadKey

Function Description:	

Arguments:				f   		- handle of char file
									str 		- pointer to current position in input string
									field		- pointer to buffer the result should be stored
									pattern	- pointer to field name pattern
									buf			- pointer to buffer a key value is to be stored
									size		- pointer to size of value buffer/ key value size

Return Value:			None

Notes:					  

*****************************************************************************************/
int
NVM_ReadKey (CHAR_FILE * file, char *str, char *field, char *pattern,
			 void *buf, UINT32 * size)
{
	int len, type = 0, ret = -1, shift = 0;
	char *p;

//	printk("NVM_ReadKey: f:%s p:%s\n",field, pattern);
	if (!OsStrCmp (field, pattern)) {
		if ((str = NVM_GetField (file, str, field)) != NULL) {
			if ((len = OsStrLen (field)) != 0) {
				type = simple_strtoul (field, &p, 16);
//              printk("NVM_ReadKey: '%s' type: %x\n", pattern, type);
			}
//          printk("NVM_ReadKey: key:%s type:%x :\n",field, type);
			if (type & 0x00000001) {
// raw data
				for (len = 0; (len < *size) && (str != NULL); len++) {
					str = NVM_GetField (file, str, field);
					ret = simple_strtoul (field, &p, 16);
					if (type & 0x00010000) {
						*((UINT32 *) buf) =
							(*((UINT32 *) buf) & (0xFFFFFFFFL << shift)) | (ret
																		  <<
																		  shift);
						shift += 8;
						if (!(shift & 0x78)) {
//                      printk("%8x ", (UINT32 *)buf);
							((UINT32 *) buf)++;
						}
					} else {
						*((char *) buf)++ = (unsigned char) ret;
//                      printk("%2x ", (unsigned char)ret);
					}
				}
//              printk("%d %d %p", len, *size, str);
				*size = len;
				ret = 0;
			} else if (!type) {
// string
				str = NVM_GetField (file, str, field);
				if (*field++ == '\"') {
					if ((p = strchr (field, '\"')) != NULL) {
						*p = '\0';
					} else {
						printk ("NVM: String field format error (near \"%s\"\n", field);
						*field = '\0';
					}
				} else
					*field = '\0';
				ret = 0;
				*size = min (strlen (field), (unsigned) *size);
				OsStrCpy (buf, field);
//              printk("'%s'", buf);
			} else {
				printk ("NVM: Unknown field type\n");
			}
//          printk("\n");
		} else {
			printk ("NVM: syntax error\n");
		}
	}
	return ret;
}


/****************************************************************************************

Function Name:			NVM_ReadCountry

Function Description:	Read country paramters from Registry

Arguments:				hNVM - NVM handle
						wCountry -  Country T.35 code
						pStruct - Pointer to the coutnry parameters

Return Value:			COM_STATUS_SUCCESS if ok; otherwise, COM_STATUS_FAIL.

Notes:

*****************************************************************************************/
//extern CtryPrmsStruct CountryProfile;
#ifdef LINUX_NVM
char *
NVM_GetSubKey (CHAR_FILE * f, char *subkey)
{
	char *p, *ret = NULL, str[80];

	while (1) {
		p = NVM_gets (f);
		if (p != NULL) {
// Get first field in string        
			if ((p = NVM_GetField (f, p, str)) == NULL) {
				if (*str != '[')
					continue;
//              printk("NVM_GetSubKey: No such key in the section\n");
				break;
			}
// Is the field "HKR"   
			if (OsStrCmp (str, "HKR"))
				continue;
// Get next field   
			if ((p = NVM_GetField (f, p, str)) == NULL)
				continue;

			strcpy(subkey, str);
//          printk("%s\n", p);
			ret = p;
		}
		break;
	}
	return ret;
}
static COM_STATUS
NVM_ReadCountry (HANDLE hNVM, UINT16 wCountryT35, CtryPrmsStruct * pStruct)
{
	char subkey[256];
	char field[80];
	char idx[80];
	char *str;
	UINT32 dwSize;
	COM_STATUS cs = COM_STATUS_FAIL;

//	printk("NVM_ReadCountry: %x\n",(unsigned int)wCountryT35);

	if (!hNVM) {
		goto err;
	}

	if (!(((NVM_HANDLE *) hNVM)->cty_file.file = OsFOpen (cty_name, "r"))) {
		goto err;
	}

	NVM_rewind (&((NVM_HANDLE *) hNVM)->cty_file, 0L);
	if (!NVM_FindSection
		(&((NVM_HANDLE *) hNVM)->cty_file,
		 &((NVM_HANDLE *) hNVM)->sectionList[Cty_Section])) {
		goto errclose;
	}

	idx[0] = '\0';
	OsMemSet(pStruct, 0, sizeof(*pStruct));

	while ((str = NVM_GetSubKey (&((NVM_HANDLE *) hNVM)->cty_file,
								subkey)) != NULL) {

		if(strncmp(subkey, "Country\\", 8))
			continue;

		if(strcmp(idx, subkey + 8)) {

//			printk(__FUNCTION__": looking for %x, got %x\n", (unsigned int)wCountryT35, (unsigned int)pStruct->T35Code);
			if(pStruct->T35Code == wCountryT35) {
				/* found it */
				break;
			}

			strncpy(idx, subkey + 8, sizeof(idx));
			OsMemSet(pStruct, 0, sizeof(*pStruct));
		}

		if ((str = NVM_GetField (&((NVM_HANDLE *) hNVM)->cty_file, str,
						   field)) == NULL) {
			continue;
		}

		dwSize = sizeof (pStruct->Blacklisting);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "BLACKLISTING", &pStruct->Blacklisting, &dwSize);
		dwSize = sizeof (pStruct->Cadence);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "CADENCE", &pStruct->Cadence, &dwSize);
		dwSize = sizeof (pStruct->CallerID);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "CALLERID", &pStruct->CallerID, &dwSize);
		dwSize = sizeof (pStruct->CallerID2);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "CALLERID2", &pStruct->CallerID2, &dwSize);
		dwSize = sizeof (pStruct->CallWaitingParms);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "CALLWAITING_PARMS", &pStruct->CallWaitingParms, &dwSize);
		dwSize = sizeof (pStruct->DTMF);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "DTMF", &pStruct->DTMF, &dwSize);
		dwSize = sizeof (pStruct->Filter);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "FILTER", &pStruct->Filter, &dwSize);
		dwSize = sizeof (pStruct->Flags);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "FLAGS", &pStruct->Flags, &dwSize);
		dwSize = sizeof (pStruct->cIntCode);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "INTCODE", &pStruct->cIntCode, &dwSize);
		dwSize = sizeof (pStruct->cInter);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "NAME", &pStruct->cInter, &dwSize);
		dwSize = sizeof (pStruct->Pulse);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "PULSE", &pStruct->Pulse, &dwSize);
		dwSize = sizeof (pStruct->Relays);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "RELAYS", &pStruct->Relays, &dwSize);
		dwSize = sizeof (pStruct->Ring);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "RING", &pStruct->Ring, &dwSize);
		dwSize = sizeof (pStruct->RLSD);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "RLSD", &pStruct->RLSD, &dwSize);
		dwSize = sizeof (pStruct->AgressSpeedIndex);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "SPEEDADJUST", &pStruct->AgressSpeedIndex, &dwSize);
		dwSize = sizeof (pStruct->SRegLimits);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "SREG", &pStruct->SRegLimits, &dwSize);
		dwSize = sizeof (pStruct->T35Code);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "T35CODE", &pStruct->T35Code, &dwSize);
		dwSize = sizeof (pStruct->Threshold);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "THRESHOLD", &pStruct->Threshold, &dwSize);
		dwSize = sizeof (pStruct->Timing);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "TIMING", &pStruct->Timing, &dwSize);
		dwSize = sizeof (pStruct->Tone);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "TONE", &pStruct->Tone, &dwSize);
		dwSize = sizeof (pStruct->Txlevel);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "TXLEVEL", &pStruct->Txlevel, &dwSize);
		dwSize = sizeof (pStruct->V92Control);
		NVM_ReadKey (&((NVM_HANDLE *) hNVM)->cty_file, str, field,
					 "V92_CONTROL", &pStruct->V92Control, &dwSize);

	}

//	printk(__FUNCTION__": done looking for %x, got %x, Relays[0]=0x%x\n", (unsigned int)wCountryT35, (unsigned int)pStruct->T35Code, (unsigned int)pStruct->Relays[0]);
	if(pStruct->T35Code == wCountryT35) {
		cs = COM_STATUS_SUCCESS;
	}

errclose:
	OsFClose (((NVM_HANDLE *) hNVM)->cty_file.file);
	((NVM_HANDLE *) hNVM)->cty_file.file = NULL;

err:
//  printk("NVM_ReadCountry: %x - No such country - restoring defaults\n",wCountry);
//  OsMemCpy(pStruct, (void*)&CountryProfile, sizeof(CtryPrmsStruct));
	return cs;
}

#else

static COM_STATUS
NVM_ReadCountry (HANDLE hNVM, UINT16 wCountry, CtryPrmsStruct * pStruct)
{
	TRACE (T__NVMMGR,
		   ("NVM_Read(CFGMGR_COUNTRY_STRUCT) - restoring default\n"));
	OsMemCpy (pStruct, (void *) &CountryProfile, sizeof (CtryPrmsStruct));
	return COM_STATUS_SUCCESS;
}
#endif

#define REG_SZ			0x00000000
#define REG_DWORD		0x00010001
#define REG_BINARY	0x00000001


typedef struct tagRegTagName {
	CFGMGR_CODE eCode;
	UINT32 dwType;
	INF_SECTION pSection;
	char *szSubKey;
	char *szName;
} REG_TAG_NAME;

static REG_TAG_NAME RegTagName[] = {
	{CFGMGR_COUNTRY_CODE, REG_DWORD, Country_Section, "Country", "Current"},
	{CFGMGR_PREVIOUS_COUNTRY_CODE, REG_DWORD, Country_Section, "Country", "Previous"},
	{CFGMGR_COUNTRY_CODE_LIST, REG_SZ, Country_Section, "Country", "CountryList"},
	{CFGMGR_COUNTRY_NAME, REG_SZ, Country_Section, "Country", "?CountryName?"},
	{CFGMGR_COUNTRY_BLIST, REG_SZ, Country_Section, "Country", "?CountryBList?"},
	{CFGMGR_BLACK_LIST, REG_SZ, Country_Section, "Country", "?BlackList?"},
	{CFGMGR_PROFILE_STORED, REG_BINARY, OEM_Section, "OEM", "Profile"},
	{CFGMGR_POUND_UD, REG_BINARY, OEM_Section, "OEM", "PoundUD"},
	{CFGMGR_ATMESSAGE_I0, REG_SZ, OEM_Section, "OEM\\ATI", "0"},
	{CFGMGR_ATMESSAGE_I1, REG_SZ, OEM_Section, "OEM\\ATI", "1"},
	{CFGMGR_ATMESSAGE_I2, REG_SZ, OEM_Section, "OEM\\ATI", "2"},
	{CFGMGR_ATMESSAGE_I3, REG_SZ, OEM_Section, "OEM\\ATI", "3"},
	{CFGMGR_ATMESSAGE_I4, REG_SZ, OEM_Section, "OEM\\ATI", "4"},
	{CFGMGR_ATMESSAGE_I5, REG_SZ, OEM_Section, "OEM\\ATI", "5"},
	{CFGMGR_ATMESSAGE_I6, REG_SZ, OEM_Section, "OEM\\ATI", "6"},
	{CFGMGR_ATMESSAGE_I7, REG_SZ, OEM_Section, "OEM\\ATI", "7"},
	{CFGMGR_ATMESSAGE_I8, REG_SZ, OEM_Section, "OEM\\ATI", "8"},
	{CFGMGR_ATMESSAGE_I9, REG_SZ, OEM_Section, "OEM\\ATI", "9"},
	{CFGMGR_ATMESSAGE_I10, REG_SZ, OEM_Section, "OEM\\ATI", "10"},
	{CFGMGR_ATMESSAGE_I11, REG_SZ, OEM_Section, "OEM\\ATI", "11"},
	{CFGMGR_ATMESSAGE_I12, REG_SZ, OEM_Section, "OEM\\ATI", "12"},
	{CFGMGR_OEM_FLAGS, REG_BINARY, Modem_Section, "OEM", "FLAGS"},
	{CFGMGR_OEM_FLAGS2, REG_BINARY, Modem_Section, "OEM", "FLAGS2"},
	{CFGMGR_OEM_FILTERS, REG_BINARY, OEM_Section, "OEM", "FILTER"},
	{CFGMGR_OEM_SPKR_MUTE_DELAY, REG_BINARY, OEM_Section, "OEM", "SPKR_MUTE_DELAY"},
	{CFGMGR_SMARTDAA, REG_BINARY, OEM_Section, "OEM", "SmartDAAParams"},
//    {CFGMGR_OEM_READ_MASK,              REG_BINARY,     "OEM",          "???"           },
	{CFGMGR_OEM_THRESHOLD, REG_BINARY, OEM_Section, "OEM", "?MOD_THRESHOLD?"},
	{CFGMGR_RING_PARAMS, REG_BINARY, OEM_Section, "OEM", "?RING?"},
	{CFGMGR_DIALTONE_WAIT_TIME, REG_BINARY, OEM_Section, "OEM", "?DIALTONE_WAIT_TIME?"},
	{CFGMGR_CALL_PROGRESS_FLAGS, REG_BINARY, OEM_Section, "OEM", "?CALL_PROGRESS_FLAGS?"},
	{CFGMGR_CALL_PROGRESS_TIMING, REG_BINARY, OEM_Section, "OEM", "?CFGMGR_CALL_PROGRESS_TIMING?"},
	{CFGMGR_SREG_LIMITS, REG_BINARY, OEM_Section, "OEM", "?SREGS?"},
	{CFGMGR_PR_IGNORE_TIME, REG_BINARY, OEM_Section, "OEM", "?PR_IGNORE_TIME?"},
//    {CFGMGR_CID_TYPE,                         REG_BINARY,  OEM_Section,         "OEM",          "?CID_TYPE?" },
	{CFGMGR_FILTERS, REG_BINARY, OEM_Section, "OEM", "?FILTERS?"},
	{CFGMGR_DTMF, REG_BINARY, OEM_Section, "OEM", "?DTMF?"},
	{CFGMGR_RLSDOFFSET, REG_BINARY, OEM_Section, "OEM", "?RLSDOFFSET?"},
	{CFGMGR_THRESHOLD, REG_BINARY, OEM_Section, "OEM", "?THRESHOLD?"},
	{CFGMGR_RELAYS, REG_BINARY, OEM_Section, "OEM", "?RELAYS?"},
	{CFGMGR_TXLEVELS, REG_BINARY, OEM_Section, "OEM", "?TXLEVELS?"},
	{CFGMGR_SPEED_ADJUST, REG_BINARY, OEM_Section, "OEM", "?SPEED_ADJUST?"},
	{CFGMGR_PULSE_DIAL_CONFIG, REG_BINARY, OEM_Section, "OEM", "?PULSE_DIAL_CONFIG?"},
	{CFGMGR_CALL_WAITING, REG_BINARY, OEM_Section, "OEM\\DEFAULT", "CW"},
	{CFGMGR_PCMUPSTREAM, REG_BINARY, OEM_Section, "OEM\\DEFAULT", "PIG"},
	{CFGMGR_MOH, REG_BINARY, OEM_Section, "OEM\\DEFAULT", "MOH"},
//    {CFGMGR_CID_TYPE,                         REG_BINARY,  OEM_Section,         "OEM",          "???" },

#ifdef VOICE_SUPPORT
	{CFGMGR_VOICE_CACHED_MISC_PARAMS, REG_BINARY, OEM_Section, "OEM", "MISC_VOICE_PARMS"},
#endif
	{CFGMGR_LAST, 0, 0, 0, 0},
};


/****************************************************************************************

Function Name:			NVM_ConvertCode2Key

Function Description:	Convert Configuration Manager enumerated code type to a
						Registry sub key.

Arguments:				dstSubKey - Pointer to the sub key
						dstName -  Pointer to the key name
						pdwType - Sub key type
						srcCode - Configuration Manager code

Return Value:			COM_STATUS_SUCCESS if ok; otherwise, COM_STATUS_FAIL.

Notes:					

*****************************************************************************************/
static COM_STATUS
NVM_ConvertCode2Key (INF_SECTION * dstSection, char *dstSubKey, char *dstName,
					 PUINT32 pdwType, CFGMGR_CODE srcCode)
{

	int i;

//      printk("NVM_ConvertCode2Key\n");
	for (i = 0;; i++) {
		if (CFGMGR_LAST == RegTagName[i].eCode)
			return COM_STATUS_FAIL;

		if (srcCode == RegTagName[i].eCode) {
			*dstSection = RegTagName[i].pSection;
			OsStrCpy (dstSubKey, RegTagName[i].szSubKey);
			OsStrCpy (dstName, RegTagName[i].szName);
			*pdwType = RegTagName[i].dwType;
			break;
		}
	}

//	printk("NVM_ConvertCode2Key: %d %s %s\n", srcCode, dstSubKey, dstName);
	return COM_STATUS_SUCCESS;
}


/****************************************************************************************

Function Name:			NVM_Read

Function Description:	Read parameters from Registry designated by eCode.

Arguments:				hNVM - NVM handle
						eCode - Configuration Manager parameter type.
						pBuf - pointer to the parameter buffer.
						pdwSize - Size of buffer.

Return Value:			COM_STATUS_SUCCESS

Notes:					

*****************************************************************************************/
extern OEMFlagStructure OEM_Flags;
COM_STATUS
NVM_Read (HANDLE hNVM, CFGMGR_CODE eCode, PVOID pBuf, UINT32 * pdwSize)
{

	COM_STATUS cs = COM_STATUS_SUCCESS;
	char szSubKey[128], szName[128], *str, field[80];
	UINT32 dwType;
	INF_SECTION pSection;

	if (hNVM == NULL) {
		printk ("NVM_Read: ERROR!!! - NULL handle\n");
		cs = COM_STATUS_FAIL;
	} else {
	  again:
		switch (eCode) {
			// special case - country parameters
		case CFGMGR_COUNTRY_STRUCT:
//			printk ("NVM_Read(CFGMGR_COUNTRY_STRUCT)\n");
			return NVM_ReadCountry (hNVM, (UINT32) pdwSize, pBuf);
			// special case - hardcoded Factory Default Profile
/*    case CFGMGR_OEM_FLAGS:
	    OsMemCpy(pBuf, (void*)&OEM_Flags, *pdwSize);
      return COM_STATUS_SUCCESS;
*/
		case CFGMGR_PROFILE_FACTORY:
//    case CFGMGR_PROFILE_STORED:
			{
//				printk ("NVM_Read(CFGMGR_PROFILE_FACTORY) - restoring default\n");
				*pdwSize = min ((unsigned) *pdwSize, sizeof (PROFILE_DATA));
				OsMemCpy (pBuf, (void *) &g_FactoryProfile, *pdwSize);
				return COM_STATUS_SUCCESS;
			}
		default:
//			printk ("NVM_Read:%d(%ld) - ", eCode, *pdwSize);
			if (COM_STATUS_SUCCESS !=
				NVM_ConvertCode2Key (&pSection, szSubKey, szName, &dwType,
									 eCode)) {
//				printk (" UNKNOWN\n");
				return COM_STATUS_FAIL;
			}
//			printk ("sec:%s sub:%s key:%s type:%lx\n",
//					((NVM_HANDLE *) hNVM)->sectionList[pSection].name,
//					szSubKey, szName, dwType);

			if (!
				(((NVM_HANDLE *) hNVM)->inf_file.file =
				 OsFOpen (inf_name, "r"))) {
				printk ("NVM_Read: ERROR - CAN'T OPEN FILE %s\n", inf_name);
				return COM_STATUS_FAIL;
			}

			NVM_rewind (&((NVM_HANDLE *) hNVM)->inf_file, 0L);
			cs = COM_STATUS_FAIL;
			if (NVM_FindSection
				(&((NVM_HANDLE *) hNVM)->inf_file,
				 &(((NVM_HANDLE *) hNVM)->sectionList[pSection])))
				while ((str = NVM_FindSubKey (&((NVM_HANDLE *) hNVM)->inf_file,
										szSubKey)) != NULL) {
					if ((str = NVM_GetField (&((NVM_HANDLE *) hNVM)->inf_file, str, field)) == NULL) {
						continue;
					}
					if (!NVM_ReadKey (&((NVM_HANDLE *) hNVM)->inf_file, str, field, szName, pBuf, pdwSize)) {
						cs = COM_STATUS_SUCCESS;
						break;
					}
				}
			OsFClose (((NVM_HANDLE *) hNVM)->inf_file.file);
			((NVM_HANDLE *) hNVM)->inf_file.file = NULL;
			if (cs != COM_STATUS_SUCCESS) {
				//printk ("NVM: No such key: %s, %s\n", szSubKey, szName);
				if (eCode == CFGMGR_PROFILE_STORED) {
					eCode = CFGMGR_PROFILE_FACTORY;
					goto again;
				}
			}
		}
	}
	return cs;
}


/****************************************************************************************

Function Name:			NVM_Write

Function Description:	Store updated parameters into the Registry.

Arguments:				hNVM - NVM handle
						eCode - Configuration Manager parameter type.
						pBuf - pointer to the parameter buffer.
						pdwSize - Size of buffer.

Return Value:			COM_STATUS_SUCCESS if ok; otherwise, COM_STATUS_FAIL.

Notes:					

*****************************************************************************************/

#if 0
COM_STATUS
NVM_Write (HANDLE hNVM, CFGMGR_CODE eCode, PVOID pBuf, PUINT32 pdwSize)
{
//     return COM_STATUS_FAIL;
	return COM_STATUS_SUCCESS;
}
#endif

int
NVM_Flush (CHAR_FILE * f)
{
//	printk("NVM_Flush\n");
	if (OsFWrite (f->buf, 1, f->size, f->file) != f->size) {
		printk ("NVM: Write ERROR!!!\n");
		return -1;
	}
	f->size = 0;
	return 0;
}

void
NVM_puts (CHAR_FILE * f, char *str)
{
	int len;
//  printk("NVM_puts: %s\n", str);
	while ((len = OsStrLen (str))) {
		len = min (CFILE_BUFFER_SIZE - f->size, len);
		OsStrnCpy (f->buf + f->size, str, len);
		str += len;
		if ((f->size += len) >= CFILE_BUFFER_SIZE) {
			if (NVM_Flush (f)) {
				break;
			}
		}
	}
}

void
NVM_WriteKey (CHAR_FILE * f, char *szSubKey, char *szName, UINT32 type,
			  void *pBuf, PUINT32 pdwSize)
{
	char s[80];
	UINT32 size;
//char comment[] = "";

	NVM_puts (f, "HKR,");
	NVM_puts (f, szSubKey);
	NVM_puts (f, ",");
	NVM_puts (f, szName);

	OsSprintf (s, ",%x,\\\n", (type == 0x10001) ? 1 : type);
	NVM_puts (f, s);
	switch (type) {
	case 0:
		NVM_puts (f, "\"");
		NVM_puts (f, pBuf);
		NVM_puts (f, "\"\n");
		break;
	case 0x00010001:
		for (size = 0; size < *pdwSize; size++) {
			UINT32 dw = 0;
			if (!(size % sizeof (UINT32))) {
				dw = *((UINT32 *) pBuf)++;
			}
			OsSprintf (s, "%2x", dw & 0xff);
			NVM_puts (f, s);
			if (size + 1 >= *pdwSize)
				NVM_puts (f, "\n");
			else
				NVM_puts (f, ",");
			dw >>= 8;
		}
/*		
			for(size = 0; size < *pdwSize; size += sizeof(UINT32))
			{
				int i;
				for(i = 0; (i < sizeof(UINT32)-1) && (size+i < *pdwSize); i++)
				{
					OsSprintf(s, "%2x,", dw & 0xff);
					NVM_puts(f, s);
					dw >>= 8;
				}
				if(size+sizeof(UINT32) >= *pdwSize)
					OsSprintf(s, "%2x\n", dw & 0xff);
				else
					OsSprintf(s, "%2x, \\\n", dw & 0xff);					
				NVM_puts(f, s);
			}
*/
		break;
	default:
		for (size = 0; size < *pdwSize - 1; size++) {
//              if((size+1) & 3)
			OsSprintf (s, "%2x,", *((unsigned char *) pBuf)++);
/*				else
					OsSprintf(s, "%2x,\\\n", *((unsigned char *)pBuf)++);*/
			NVM_puts (f, s);
		}
		OsSprintf (s, "%2x\n", *((unsigned char *) pBuf)++);
		NVM_puts (f, s);
	}
	NVM_Flush (f);
}



COM_STATUS
NVM_Write (HANDLE hNVM, CFGMGR_CODE eCode, PVOID pBuf, PUINT32 pdwSize)
{
	char szSubKey[128], szName[128], *str, field[80];
	UINT32 dwType;
	COM_STATUS cs = COM_STATUS_SUCCESS;
	int nosec = 0;
	INF_SECTION pSection;
	UINT32 curPos = 0, nextPos = 0;

	if ((CFGMGR_PROFILE_STORED != eCode) &&
		(CFGMGR_POUND_UD != eCode) &&
		(CFGMGR_PREVIOUS_COUNTRY_CODE != eCode) &&
		(CFGMGR_COUNTRY_CODE != eCode))
		return COM_STATUS_SUCCESS;

//	printk("NVM_Write:%d(%ld) - ", eCode, *pdwSize);
	if (COM_STATUS_SUCCESS !=
		NVM_ConvertCode2Key (&pSection, szSubKey, szName, &dwType, eCode)) {
//      printk("UNKNOWN\n", eCode, *pdwSize);
		return COM_STATUS_FAIL;
	}

//    printk("sec:%s sub:%s key:%s type:%x\n", ((NVM_HANDLE *)hNVM)->sectionList[pSection].name, szSubKey, szName, dwType);
//
	if (!(((NVM_HANDLE *) hNVM)->inf_file.file = OsFOpen (inf_name, "r"))) {
		printk ("NVM_Write: ERROR - CAN'T OPEN FILE %s\n", inf_name);
		return COM_STATUS_FAIL;
	}

	NVM_rewind (&((NVM_HANDLE *) hNVM)->inf_file, 0L);
//      cs = COM_STATUS_FAIL;
	if (NVM_FindSection
		(&((NVM_HANDLE *) hNVM)->inf_file,
		 &(((NVM_HANDLE *) hNVM)->sectionList[pSection]))) {
		nosec = 1;
//      printk("NVM_Write: %s section found\n", ((NVM_HANDLE *)hNVM)->sectionList[pSection].name);
		while ((str =
				NVM_FindSubKey (&((NVM_HANDLE *) hNVM)->inf_file,
								szSubKey)) != NULL) {
//          printk("NVM_Write: '%s' found at %d\n", szSubKey,curPos);
			if ((str =
				 NVM_GetField (&((NVM_HANDLE *) hNVM)->inf_file, str,
							   field)) == NULL) {
				continue;
			}
			if (!OsStrCmp (field, szName)) {
				curPos = ((NVM_HANDLE *) hNVM)->inf_file.position;
//              printk("NVM_Write: '%s,%s' found at %d\n", szSubKey, szName,curPos);
				while ((str =
						NVM_GetField (&((NVM_HANDLE *) hNVM)->inf_file, str,
									  field)) != NULL);
				NVM_gets (&((NVM_HANDLE *) hNVM)->inf_file);
				break;
			}
		}
	}

	nextPos = ((NVM_HANDLE *) hNVM)->inf_file.position;

	if (!curPos)
		curPos = nextPos;

	if (((NVM_HANDLE *) hNVM)->sectionList[Modem_Section].dwPosition >
		curPos);
	((NVM_HANDLE *) hNVM)->sectionList[Modem_Section].dwPosition = 0;
	if (((NVM_HANDLE *) hNVM)->sectionList[OEM_Section].dwPosition > curPos);
	((NVM_HANDLE *) hNVM)->sectionList[OEM_Section].dwPosition = 0;
	if (((NVM_HANDLE *) hNVM)->sectionList[Country_Section].dwPosition >
		curPos);
	((NVM_HANDLE *) hNVM)->sectionList[Country_Section].dwPosition = 0;

//    printk("NVM_Write: cur: %d next: %d\n", curPos, nextPos);
	if (!
		(((NVM_HANDLE *) hNVM)->new_file.file =
		 OsFOpen (new_name, "w"))) {
		printk ("NVM_Write: ERROR - CAN'T OPEN FILE %s\n", new_name);

		OsFClose(((NVM_HANDLE *)hNVM)->inf_file.file);
		((NVM_HANDLE *)hNVM)->inf_file.file = NULL;
		return COM_STATUS_FAIL;
	}

	NVM_rewind (&((NVM_HANDLE *) hNVM)->new_file, 0L);

	NVM_copy (&((NVM_HANDLE *) hNVM)->new_file,
			  &((NVM_HANDLE *) hNVM)->inf_file, 0, curPos);

	if (!nosec) {
		NVM_puts (&((NVM_HANDLE *) hNVM)->new_file,
				  ((NVM_HANDLE *) hNVM)->sectionList[pSection].name);
		NVM_puts (&((NVM_HANDLE *) hNVM)->new_file, "\n");
	}

	NVM_WriteKey (&((NVM_HANDLE *) hNVM)->new_file, szSubKey, szName, dwType,
				  pBuf, pdwSize);

	NVM_copy (&((NVM_HANDLE *) hNVM)->new_file,
			  &((NVM_HANDLE *) hNVM)->inf_file, nextPos, 0xFFFFFFFF);

	// NVM_SubstFile closes both files
	NVM_SubstFile (&((NVM_HANDLE *) hNVM)->inf_file,
				   &((NVM_HANDLE *) hNVM)->new_file);
//	OsFClose(((NVM_HANDLE *)hNVM)->inf_file.file);
//	((NVM_HANDLE *)hNVM)->inf_file.file = NULL;
//	((NVM_HANDLE *)hNVM)->sectionList[Cty_Section].dwPosition = 0;
	return cs;
}
