/*
 * Copyright (c) 2003 Linuxant inc.
 * Copyright (c) 2001-2003 Conexant Systems, Inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

/*************************************************************************

	File Name: Memory.c

	File Description:

*************************************************************************/
#include "oscompat.h"
#include "ostypedefs.h"
#include "osresour_ex.h"

#include <linux/pci.h>
#include <linux/slab.h>
#include <linux/pm.h>
#include <asm/io.h>

HANDLE OSFindPCIDevice(int vendorId, int deviceId)
{
#ifdef CONFIG_PCI
    if (!pci_present())   /* No PCI bus in this machine! */
#endif
    {
        printk(KERN_ERR "%s: No PCI bus in this machine!\n", __FUNCTION__);
        return NULL;
    }
    return pci_find_device(vendorId,
                           deviceId,
                           NULL);
}

BOOL OSGetPCIDeviceResources(HANDLE DevNode, UINT32* pdwPhysMemBase, UINT32* pdwMemSize, UINT8* pcIrq)
{			   
    struct pci_dev *pcidev = (struct pci_dev *)DevNode;
	int i;

	*pdwPhysMemBase = 0;
	*pdwMemSize = 0;
	*pcIrq = 0;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	if ( (i = pci_enable_device(pcidev)) ) {
		printk(KERN_ERR "%s: pci_enable_device() failed (err=%d)\n", __FUNCTION__, i);
		return FALSE;
	}

	for (i = 0; i < DEVICE_COUNT_RESOURCE; i++) {
		if ((pci_resource_flags(pcidev, i) & PCI_BASE_ADDRESS_SPACE) ==
				PCI_BASE_ADDRESS_SPACE_MEMORY) {
			*pdwPhysMemBase = pci_resource_start(pcidev, i) & PCI_BASE_ADDRESS_MEM_MASK;
			*pdwMemSize = pci_resource_len(pcidev, i);
			break;
		}
	}
#else
	for (i = 0;
			i < sizeof(pcidev->base_address) / sizeof(pcidev->base_address[0]);
			i++) {
		if ((pcidev->base_address[i] & PCI_BASE_ADDRESS_SPACE) ==
				PCI_BASE_ADDRESS_SPACE_MEMORY) {
			*pdwPhysMemBase = pcidev->base_address[i] & PCI_BASE_ADDRESS_MEM_MASK;
			*pdwMemSize = 64*1024;
			break;
		}
	}
#endif

	*pcIrq = pcidev->irq;

	return *pdwPhysMemBase && *pdwMemSize && *pcIrq;
}

UINT32 OSMapMemory(UINT32 dwPhysMemBase, UINT32 dwMemSize)
{
	UINT32 mem;

	mem = (UINT32)ioremap(dwPhysMemBase, dwMemSize);
	if(!mem) {
        printk(KERN_ERR "%s: ioremap(0x%lx, 0x%lx) failed\n", __FUNCTION__, dwPhysMemBase, dwMemSize);
	}
//	printk("\n<HCF> Map %p (%lu) to %p\n\n", (void *)dwPhysMemBase, dwMemSize, (void *)mem);
	return mem;
}

BOOL OSUnmapMemory(UINT32 dwLinearAddr, UINT32 dwMemSize)
{
	if((void *)dwLinearAddr != NULL)
		iounmap((void *)dwLinearAddr);
	return TRUE;
}

UINT32 OSHookInterrupt(UINT8 cIrq, UINT32 IntProc, void* RefData)
{
  if (request_irq((int)cIrq, (void*)IntProc, SA_SHIRQ, "hcf", RefData))
  {
	 printk(KERN_ERR "%s: request_irq %d failed\n", __FUNCTION__, cIrq);
	 return 0;
  }
  return (UINT32)cIrq;
}

void OSUnhookInterrupt(UINT32 dwHandle, void *RefData)
{
    if(dwHandle)
	    free_irq((int)dwHandle, RefData);
}


static int 
linux_pm_callback(struct pm_dev *dev, pm_request_t rqst, void *data) 
{
	POS_PM_CONFIG pPmConf = (POS_PM_CONFIG) dev->data;

	if ( ! pPmConf )
		return 0;

	printk(KERN_DEBUG "%s: pm event 0x%x received\n", __FUNCTION__, rqst);
	
	switch (rqst) {
		case PM_SUSPEND: 
			if(pPmConf->SuspendOKCB && !pPmConf->SuspendOKCB(pPmConf->SuspendOKInst))
				return -EBUSY;


			if(pPmConf->SuspendCB && !pPmConf->SuspendCB(pPmConf->SuspendInst))
				return -EIO;
		break;
		case PM_RESUME: 
			if(pPmConf->ResumeCB && !pPmConf->ResumeCB(pPmConf->ResumeInst))
				return -EIO;
		break;
		/*
		 * we'd also like to find out about
		 * power level changes because some biosen
		 * could do mean things to our device when they
		 * change their power state.
		 */
        }
	return 0;
}

HANDLE	OSInitPowerManagement(HANDLE DevNode, POS_PM_CONFIG pPmConf)
{
	struct pm_dev *pmdev;
	struct pci_dev *pcidev = (struct pci_dev *)DevNode;

	pmdev = pm_register(PM_PCI_DEV, PM_PCI_ID(pcidev), linux_pm_callback);
	if (pmdev) {
		pmdev->data = pPmConf;
	}

	return (HANDLE)pmdev;
}

void	OSTermPowerManagement(HANDLE pmdev)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	pm_unregister_all(linux_pm_callback);
#else
	pm_unregister((struct pm_dev *)pmdev);
#endif
}
