/*
 * Copyright (c) 2003 Linuxant inc.
 * Copyright (c) 2001-2003 Conexant Systems, Inc.
 *
 * NOTE: The use and distribution of this software is governed by the terms in
 * the file LICENSE, which is included in the package. You must read this and
 * agree to these terms before using or distributing this software.
 *
 */

/*************************************************************************

  File Name: OsString.c

  File Description: This file implements all the string/memory utilities.

  Function list:

  GLOBAL:

  static:


  ************************************************************************/

#include "oscompat.h"
#include "ostypedefs.h"

	/********************************************************************************
	
	Function Name: OsStrCpy.

	Function Description:The function copies a specified a string from one 
						 buffer to another. This function may not work correctly if 
						 the buffers overlap. 

	Arguments: BuffDest - Pointer to buffer to contain copied string.
			   BuffSrc - Pointer to buffer with the string to copy.
	
	Return Value: A pointer to BuffDest.

	********************************************************************************/
	PVOID	OsStrCpy(PVOID BuffDest, PVOID BuffSrc)
	{
/*		int index = 0;
		while (*((char *)BuffSrc+index) != '\0')
		{
			*((char *)BuffDest+index) = *((char *)BuffSrc+index);
			index++;
		}
		*((char *)BuffDest+index) = '\0';
		return (BuffDest);
		*/
		return strcpy(BuffDest, BuffSrc);
	}

	/********************************************************************************
	
	Function Name: OsStrnCpy.

	Function Description:The function copies a specified a string from one 
						 buffer to another. The function does not copy more 
						 than MaxSize bytes.The behavior of strncpy is 
						 undefined if the source and destination strings overlap

	Arguments: BuffDest - Pointer to buffer to contain copied string.
			   BuffSrc - Pointer to buffer with the string to copy.
			   MaxSize - Max chars to copy
	
	Return Value: A pointer to BuffDest.

	********************************************************************************/
	PVOID	OsStrnCpy(PVOID BuffDest, PVOID BuffSrc, int MaxSize)
	{
/*		int index = 1;
		while ((*((char *)BuffSrc+(index-1)) != '\0') && index <= MaxSize)
		{
			*((char *)BuffDest+(index-1)) = *((char *)BuffSrc+(index-1));
			index++;
		}
		if (index <= MaxSize)
			*((char *)BuffDest+(index-1)) = '\0';
		return (BuffDest);
*/
		return strncpy(BuffDest, BuffSrc, MaxSize);
	}
	/********************************************************************************
	
	Function Name: OsStrCmp.

	Function Description:The strcmp function compares string1 and string2 
			lexicographically and returns a value indicating their relationship

	Arguments: string1 ,string2 - Null terminated strings to compare
	
	Return Value: lexicographic relation of string1 to string2.

	********************************************************************************/
	int	OsStrCmp(const char *string1, char *string2)
	{
/*		int result = 0;
		int index = 0;
		char char1 = *(string1+index);
		char char2 = *(string2+index);
		while (!(char1 == '\0' && char2 == '\0'))
		{
			result += (int)(char1 - char2);
			index++;
			if (char1 != '\0')
				char1 = *(string1+index);
			if (char2 != '\0')
				char2 = *(string2+index);
		}
		return (result);
*/
		return strcmp(string1, string2);
	}
	/********************************************************************************
	
	Function Name: OsStrnCmp.

	Function Description:The strcmp function compares string1 and string2 
			lexicographically and returns a value indicating their relationship
	Arguments: string1 ,string2 - Null terminated strings to compare
						MaxSize	- Number of characters to compare
	
	Return Value: lexicographic relation of string1 to string2.

	********************************************************************************/
	int	OsStrnCmp(PVOID string1, PVOID string2, int MaxSize)
	{
/*		int result = 0;
		int index = 1;
		char char1 = *((char *)string1+(index-1));
		char char2 = *((char *)string2+(index-1));
		while ((!(char1 == '\0' && char2 == '\0')) && index <= MaxSize)
		{
			result += (int)(char1 - char2);
			index++;
			if (char1 != '\0')
				char1 = *((char *)string1+(index-1));
			if (char2 != '\0')
				char2 = *((char *)string2+(index-1));
		}
		return (result);
*/
		return strncmp(string1, string2, MaxSize);
	}

	/********************************************************************************
	
	Function Name: OsToupper.

	Function Description:converts a charecter c to upper case  

	Arguments: c - charected to be converted to upper case.
	
	Return Value: upper case charecter of c.

	********************************************************************************/
	int	OsToupper(int c)
	{
		if (c >= 'a' && c <= 'z')
			return (c - ('a' - 'A'));
		return (c);
	}
	/********************************************************************************
	
	Function Name: OsTolower.

	Function Description:converts a charecter c to lower case  

	Arguments: c - charected to be converted to lower case.
	
	Return Value: lower case charecter of c.

	********************************************************************************/
	int	OsTolower(int c)
	{
		if (c >= 'A' && c <= 'Z')
			return (c + ('a' - 'A'));
		return (c);
	}
	/********************************************************************************
	
	Function Name: OsIsDigit.

	Function tests if a charecter is a decimal digit. 

	Arguments: c - integer to test.
	
	Return Value: non zero value if c is a decimal digit('0' - '9').

	********************************************************************************/
	int	OsIsDigit(int c)
	{
		return ((c >='0' && c <= '9') ? 1 : 0);
	}

#if ( OS_TYPE != OS_LINUX )

/********************************************************************************
	
	Function Name: OSSprintfFixedArgs.

	Function Description:The OSSprintfFixedArgs function formats and stores a series of 
							characters and values in buffer. The function recieves the 
							format parameters not on the stack, but using the va_list
							parameter

	Arguments: buffer - Pointer to buffer to contain copied string.
			   format - control string.
			   StartedVaList - va_list that has been started to include the format arguments

	
	Return Value: OsSprintf returns the number of bytes stored in buffer, 
					not counting the terminating null character. 

	********************************************************************************/
int OSSprintfFixedArgs(PCHAR Dest, PCHAR format, PVOID StartedVaList)
{
	va_list			args	= *(va_list*)StartedVaList		;
	unsigned long	ulong_value		;
	const char *	string_value	;
	long			long_value		;
	char			char_value		;
	int				MaxLen			;

	int FormattedStringLen = 0; // counter of how many chars were put into Dest String

	MaxLen = MAX_FORMAT_STRING_LEN -1; // leave place for '\0'
//	va_start (args , format) ;

	while (*format != '\0')
	{
		if (*format != '%')
		   FormattedStringLen += PlaceChar (MaxLen - FormattedStringLen, 
				(char *)(Dest+FormattedStringLen), *format) ;
		else
		{
			++format ;

			switch (*format)
			{
				case '%':   FormattedStringLen += PlaceChar(MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, '%') ;
							break ;
				case 'x':
				case 'X':   ulong_value = va_arg (args,unsigned long) ;
							FormattedStringLen += PlaceHex (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, ulong_value) ;
							break ;
				case 's':
				case 'S':   string_value = va_arg (args,char *) ;
							FormattedStringLen += PlaceString (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, string_value) ;
							break ;
				case 'd':
				case 'D':   long_value = va_arg (args,long) ;
							FormattedStringLen += PlaceInt (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, long_value) ;
							break ;
				case 'c':
				case 'C':	char_value = va_arg (args , char);
							FormattedStringLen += PlaceChar (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, char_value) ;
							break ;

				default:    FormattedStringLen += PlaceChar (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, format[-1]) ;
							FormattedStringLen += PlaceChar (MaxLen - FormattedStringLen, 
								Dest+FormattedStringLen, *format) ;
							break ;
			}
		}

		++format ;
	}

	Dest[FormattedStringLen] = '\0';
	return FormattedStringLen;
//	va_end (args) ;
}


#endif


/********************************************************************************

Function Name: OsSprintf.

Function Description:The function returns the length of a null-terminated string.

Arguments: String - Pointer to buffer to contain copied string.

Return Value: int - the length of a null-terminated string.

********************************************************************************/
int	OsSprintf(PCHAR Dest, PCHAR format, ...)
{
	va_list			args			;
	int				RetVal;
	
	va_start (args , format) ;
#if ( OS_TYPE == OS_LINUX )
	RetVal = vsprintf(Dest,format,args);
#else
	RetVal = OSSprintfFixedArgs(Dest, format, &args);
#endif
	va_end (args) ;
	return RetVal;
}

	/********************************************************************************

	Function Name: OsStrLen.

	Function Description:	returns the number of characters in string, not including 
							the terminating null character
	Arguments: string - Null-terminated string


	Return Value:	returns the number of characters in string, excluding the 
					terminal NULL. No return value is reserved to indicate an error.
	********************************************************************************/
	int	OsStrLen(const char * string)
	{
/*		int index=0;
		while (*(string+index) != '\0')
			index++;
		return (index);
*/
		return strlen(string);
	}

	/**************************************** Internal Functions **************************/

#if ( OS_TYPE != OS_LINUX )
static int    PlaceHex ( int MaxLen, char *Dest, unsigned long     value)
{
   static const char    HexTable[] = "0123456789ABCDEF"  ;
	int StringLen = 0;
   int                  digit                            ;

   for (digit = 7  ;  digit >= 0  ;  --digit)
      StringLen += PlaceChar (MaxLen - StringLen, Dest+StringLen, HexTable [0xF  &  (int)(value >> (digit*4))]) ;
   return StringLen;
}
static int    PlaceInt    ( int MaxLen, char *Dest, long  value) 
{
	static const char	IntTable[] = "0123456789"  ;
	int					digit                            ;
	int					digitVal;
	int					LeadingZeros = 1;
	int					Exp;
	int					StringLen = 0;

	if ( value == 0 )
		StringLen += PlaceChar (MaxLen, Dest, '0') ;
	if (value < 0)
	{
		value *= -1;
		StringLen += PlaceChar(MaxLen, Dest, '-');
	}
	for (digit = 9 , Exp = Exp10(digit);  digit >= 0  ;  --digit)
	{
		digitVal = (int)value /Exp;
		value -= digitVal * Exp;
		Exp /= 10;
		if ( (digitVal == 0) && (LeadingZeros == 1) )
			continue;
		LeadingZeros = 0;
		StringLen += PlaceChar (MaxLen - StringLen, Dest+StringLen, IntTable [digitVal]) ;
	}
	return StringLen ;
}
static int    PlaceString ( int MaxLen, char *Dest, const char *    str)
{
	int StringLen = 0;
	while (*str != '\0')
	{
		StringLen += PlaceChar (MaxLen - StringLen, Dest+StringLen, *str) ;
		str++;
	}
	return StringLen;
}
static int    PlaceChar ( int MaxLen, char *Dest, char    value)
{
	if ( MaxLen <1 )
		return 0;
	*Dest = value;
	return 1;
}
static int	   Exp10( int i)
{
	int j;
	int RetVal = 1;

	for (j = 0 ;  j <i ; j ++ )
		RetVal *= 10;
	return RetVal;
}
#endif
